@@ -5,6 +5,6 @@ mod xmltv;
 mod xmltv_error;
 
 pub use channels::{XMLTVChannel, XMLTVChannelDisplayName};
-pub use programs::{XMLTVProgram, XMLTVProgramTitle};
+pub use programs::*;
 pub use xmltv::XMLTV;
 pub use xmltv_error::{Result, XMLTVError};
